/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package Plotter;

import java.awt.*;
import java.lang.Math;

//for now, the distance between two points is 5 pixels

public class Plotterfixed extends Canvas
{
  //constants
  private static final int margin = 20;
  private static final int x_lmargin = 30;
  private static final double y_ticks = 20.0;
  private static final int frame_width = 350;
  private static final int frame_height = 350;
  //hidden data
  private int xscale;
  private double yscale;
  private double points[];
  private int numPoints;
  private int delta;
  private Dimension prefSize;
  private int x_length;
  private int y_length;
  private double maxpoint;
  private double minpoint;
  private boolean plotOnlyPoints;

  public Plotterfixed(double newPoints[], int num, int d)
  {
	reset(newPoints, num, d);
  }

  public void resetpoints(double newPoints[], int num)
  {
    for (int i = 0; i < num; i++)
      points[i] = newPoints[i];
  }
  
  public void reset(double newPoints[], int num, int d)
  {
    x_length = frame_width - margin - x_lmargin;
    y_length = frame_height - 2 * margin;
    xscale = x_length / (num - 1); //the number of pixels between points
    points = new double[num];
    prefSize = new Dimension();
    
    for (int i = 0; i < num; i++)
      points[i] = newPoints[i];
    numPoints = num;
    delta = d;
    yscale = ((double)y_length) / (max(points) - min(points));
    prefSize.width = frame_width;
    prefSize.height = frame_height;
    maxpoint = max(points);
    minpoint = min(points);
    plotOnlyPoints = false;
  }

  public double max(double a[])
  {
    double temp = a[0];
    int i = 0;
    
    for (i = 0; i < numPoints; i++)
      if (temp < a[i])
	    temp = a[i];
    return temp;
  }

  public double min(double a[])
  {
    double temp = a[0];
    int i = 0;
    
    for (i = 0; i < numPoints; i++)
      if (temp > a[i])
	temp = a[i];
    return temp;
  }
  
public void paint(Graphics  g)
  {
    int i;
    String tempstr;
    double base = Math.floor(minpoint);
    double ydelt = (maxpoint - minpoint) / y_ticks;
    int occ_x = 0; //the x position of the next available pixel
    FontMetrics metrics;

	metrics = g.getFontMetrics();
    for (i = 1; i < numPoints; i++)
    {
		tempstr = Integer.toString(delta*i);
		if (x_lmargin + xscale*i - 
		    (double)metrics.stringWidth(tempstr)/2 >= occ_x)
		{
			g.drawString(tempstr, x_lmargin + xscale*i - 
			     (metrics.stringWidth(tempstr))/2, 
			     frame_height - margin + metrics.getAscent());
			occ_x = x_lmargin + xscale*i + 
			        metrics.stringWidth(tempstr)/2 + 3;
		}
	}

    for (i = 0; i < y_ticks; i++)
    {
		g.drawString(trunc(base + (y_ticks - i)*ydelt), 0, 
		     (int) (i*(y_length / y_ticks) + margin + 5));
    }
    //draw the y-mark closest to the origin
    g.drawString(trunc(minpoint), 0, 
		 y_length + margin + 5);
    
    //draw the steenkeeng points & axis
    DrawAxis(g);
    PlotPoints(g);
  }
  
  private void DrawAxis(Graphics g)
  {
  	int i;
  	//x axis
    g.drawLine(0+x_lmargin, frame_height - margin, 
	       x_length + x_lmargin,  frame_height - margin);
    //draw the x-axis ticks
    for (i = 1; i < numPoints; i++)
    {
	  g.drawLine(x_lmargin + xscale*i, frame_height - margin,
			 x_lmargin + xscale*i, frame_height - margin - 5);
	}
    
    //y axis
    g.drawLine(0+x_lmargin, 0+margin, 0+x_lmargin, margin + y_length);
      //draw the y-axis ticks
    for (i = 0; i < y_ticks; i++)
    {
	  g.drawLine(x_lmargin, (int) (i* (y_length / y_ticks) + margin),
	  	   x_lmargin+5, (int) (i*(y_length / y_ticks) + margin));
    }
  }

  private void PlotPoints(Graphics g)
  {
    int i, p;
    double base = Math.floor(minpoint), y;
    Color save = g.getColor();
    int xcoord, prevx, prevy;
    
    g.setColor(Color.red);
    
    
    y = points[0] - base;
//y is now the distance from the x-axis to the current point
    y *= yscale; //convert to pixels
    prevy = y_length + margin - (int)Math.round(y);
    prevx = x_lmargin;
    g.fillRect(prevx, prevy, 2, 2);
    
    for (i = 1; i < numPoints; i++)
      {
	y = points[i] - base; //y is now the distance from 
			      //the x-axis to the current point
	y *= yscale; //convert to pixels
	p = y_length + margin - (int)Math.round(y);
	xcoord = x_lmargin + xscale * i;
	g.setColor(Color.red);
	g.fillRect(xcoord, p, 2, 2);
	g.setColor(Color.black);
	g.drawLine(prevx, prevy, xcoord, p);
	prevx = xcoord;
	prevy = p;
      }
    
    g.setColor(save);
  }
  
  public void setPaintMethod(short type)
  {
  	switch (type)
  	{
  	  case 0:
  	  	plotOnlyPoints = false;
  	  	break;
  	  case 1:
  	  	plotOnlyPoints = true;
  	  	break;
  	}
  }

  public void update(Graphics g)
    {
      Rectangle bob = g.getClipRect();
      if (plotOnlyPoints) {
	g.clearRect(x_lmargin, 0+margin, x_length,y_length + 2);
	DrawAxis(g);
	PlotPoints(g);
      }
      else {
	g.setColor(getBackground());
	g.fillRect(0, 0, size().width, size().height);
	g.setColor(Color.black);
	paint(g);
      }
    }

  public void repaint()
    {
      Graphics g = getGraphics();
      if (g == null)
	super.repaint();
      else
	update(getGraphics());
    }
	
  private String trunc(double d)
  {
    String temp = Double.toString(d);
    char retarray[] = new char[4];
    int i, j, power = (int) d;
    boolean haspoint = false;
    
    retarray[0] = '.';
    for (i = 1; i < 4; i++)
      retarray[i] = '0';
    if (d == 0.0)
      return new String(retarray);
    
    for (i = 0; (i < temp.length()) && (temp.charAt(i) == '0'); i++)
  	  ; //i points to the first non-zero position
    
    try {
      for (j = i; j < i + 4; j++)
	retarray[j-i] = temp.charAt(j);
    } catch (StringIndexOutOfBoundsException e){
    }
  	for (j = 0; (j < 4) && !haspoint; j++)
  	  if (retarray[j] == '.')
  	    haspoint = true;
    if (!haspoint)
      {
    	//place the decimal point
    	j = 0;
	while (power/10 != 0)
  	  {
  	    power /= 10;
  	    j++;
  	  }
  	  retarray[j + 1] = '.';
      }
    return new String(retarray);
  }

  public Dimension minimumSize() {
    return prefSize;
  }

  public Dimension preferredSize() {
    return prefSize;
  }
}
